/// <reference path="../types/all.d.ts" />
'use strict';
var debug = require('debug');
var broccoli_typescript_1 = require('./broccoli-typescript');
var ts = require('typescript');
var find_config_1 = require('./find-config');
var fs = require('fs');
/**
 * Convert from a hash (possibly loaded from file) to Typescript Compiler Options.
 * The rationale for making this available separately is to be able to handle errors
 * in the configuration data.
 */
function toTypescriptOptions(options, basePath) {
    return ts.convertCompilerOptionsFromJson(options, basePath);
}
exports.toTypescriptOptions = toTypescriptOptions;
/**
 * Load the given tsconfig.json file.
 */
function loadTsconfig(tsconfigPath) {
    if (tsconfigPath) {
        var content = JSON.parse(fs.readFileSync(tsconfigPath).toString('utf8'));
        return ts.convertCompilerOptionsFromJson(content['compilerOptions'], tsconfigPath);
    }
    return {
        options: undefined, errors: [{
                file: null,
                start: undefined,
                length: undefined,
                messageText: "Cannot load file tsconfig.json",
                category: 2,
                code: -1,
            }]
    };
}
/**
 * Load the configuration from the tsconfig.json file for the consuming project.
 */
function loadProjectTsconfig(root) {
    var path = find_config_1.default(root);
    return loadTsconfig(path);
}
exports.loadProjectTsconfig = loadProjectTsconfig;
function Compiler(tree, options) {
    var resolvedOptions = options;
    if (!options || options.tsconfig) {
        var loaded = (options && options.tsconfig) ? loadTsconfig(options.tsconfig) : loadProjectTsconfig(__dirname);
        if (loaded.errors.length) {
            throw "Errors loading tsconfig " + loaded.errors.join("\n");
        }
        resolvedOptions = { tsOptions: loaded.options };
    }
    debug('broccoli-typify')("Intializing compiler with " + JSON.stringify(resolvedOptions));
    return broccoli_typescript_1.Compiler(tree, resolvedOptions);
}
exports.Compiler = Compiler;
